﻿namespace TabsStudioAfterClose
{
    public class AfterClose : TabsStudioExt.ITabsStudioAddin
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            tabs = new Tabs(engine_, new AddinsCommon.VsVersion(dte).IsWpfShell());
            commands = new CommandEventsListener(dte);
            
            engine.ClosingSingleTab += OnClosingSingleTab;
            engine.ClosingSingleTabExtension += OnClosingSingleTabExtension;

            commands.ClosingSingleTabBefore += OnClosingSingleTabCommandBefore;
            commands.ClosingSingleTabExtensionBefore += OnClosingSingleTabExtensionCommandBefore;
            commands.ClosingSingleTabAfter += OnClosingCommandAfter;
            commands.ClosingSingleTabExtensionAfter += OnClosingCommandAfter;
        }

        public void OnDisconnection()
        {
            engine.ClosingSingleTab -= OnClosingSingleTab;
            engine.ClosingSingleTabExtension -= OnClosingSingleTabExtension;

            commands.ClosingSingleTabBefore -= OnClosingSingleTabCommandBefore;
            commands.ClosingSingleTabExtensionBefore -= OnClosingSingleTabExtensionCommandBefore;
            commands.ClosingSingleTabAfter -= OnClosingCommandAfter;
            commands.ClosingSingleTabExtensionAfter -= OnClosingCommandAfter;
        }

        private void OnClosingSingleTab(object sender, TabsStudioExt.TabEventArgs e)
        {
            Tab selectedTab = tabs.GetSelectedTab();
            if (selectedTab == null || !selectedTab.IsEqual(e.Tab.TabItem))
                return;

            ActivateNextTab(selectedTab);
        }

        private void OnClosingSingleTabCommandBefore(object sender, System.EventArgs e)
        {
            windowToActivate = null;

            Tab selectedTab = tabs.GetSelectedTab();
            if (selectedTab == null)
                return;

            windowToActivate = FindActivateNextTab(selectedTab);
        }

        private void OnClosingSingleTabExtension(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            Tab selectedTab = tabs.GetSelectedTab();
            if (selectedTab == null || !selectedTab.IsEqual(e.Tab.TabItem))
                return;

            if (selectedTab.Count() == 1)
            {
                ActivateNextTab(selectedTab);
                return;
            }

            if (e.TabExtension.IsExtensionActive)
                ActivateNextExtension(selectedTab);
        }

        private void OnClosingSingleTabExtensionCommandBefore(object sender, System.EventArgs e)
        {
            windowToActivate = null;

            Tab selectedTab = tabs.GetSelectedTab();
            if (selectedTab == null)
                return;

            if (selectedTab.Count() == 1)
            {
                windowToActivate = FindActivateNextTab(selectedTab);
                return;
            }

            windowToActivate = FindActivateNextExtension(selectedTab);
        }

        private void OnClosingCommandAfter(object sender, System.EventArgs e)
        {
            if (windowToActivate != null)
            {
                windowToActivate.Activate();
                windowToActivate = null;
            }
        }

        // REQUIRE(selectedTab.Count() > 1)
        private void ActivateNextExtension(Tab selectedTab)
        {
            FindActivateNextExtension(selectedTab).Activate();
        }

        // REQUIRE(selectedTab.Count() > 1)
        private Extension FindActivateNextExtension(Tab selectedTab)
        {
            return OffsetActiveExtension(selectedTab);
        }

        private void ActivateNextTab(Tab selectedTab)
        {
            Extension e = FindActivateNextTab(selectedTab);
            if (e != null)
                e.Activate();
        }

        private Extension FindActivateNextTab(Tab selectedTab)
        {
            if (tabs.Count() > 1)
                return OffsetTab(selectedTab).GetActiveExtension();
            return null;
        }

        // REQUIRE(tabs.Count() > 1)
        private Tab OffsetTab(Tab tab)
        {
            uint i = tabs.GetTabIndex(tab);
            return tabs.GetTab(i < tabs.Count() - 1 ? i + 1 : i - 1);
        }

        // REQUIRE(tab.Count() > 1)
        private Extension OffsetActiveExtension(Tab tab)
        {
            uint i = tab.GetExtensionIndex(tab.GetActiveExtension());
            return tab.GetExtension(i < tab.Count() - 1 ? i + 1 : i - 1);
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Tabs tabs;
        private CommandEventsListener commands;
        private Extension windowToActivate;
    }
}
